;; ZeroAtt                                   ;;
;; Lee Mac  ~  03.03.10                      ;;
;; Sets Attribute Rotation to zero upon      ;;
;; block insertion, copy, mirror.            ;;
;; Type 'ZeroAtt' to Activate and Deactivate ;;

(defun c:ZeroAtt (/ Reac *ZeroLastEnt*)
  (vl-load-COM)

  (if (setq Reac
        (vl-some
          (function
            (lambda (reactor)
              (if (eq "Zero-Att"
                    (vlr-data reactor)) reactor)))

          (cdar (vlr-reactors :vlr-command-reactor))))

    (if (vlr-added-p Reac)
      
      (vlr-remove Reac)
      (vlr-add Reac))

    (setq Reac
      (vlr-command-reactor "Zero-Att"
        (list
          (cons :vlr-commandWillStart 'GetCommand)
          (cons :vlr-commandEnded     'ZeroAttribs)))))

  (if (vlr-added-p Reac)
    (princ "\n** ZeroAtt Reactor Activated **")
    (princ "\n** ZeroAtt Reactor Deactivated **"))

  (princ))


(defun GetCommand  (Reactor Args)
  (setq *ZeroLastEnt*
    (if (wcmatch (strcase (car Args)) "*COPY,*MIRROR")
      (entlast)))
  
  (princ))


(defun ZeroAttribs (Reactor Args / *error*
                                   GetLocked PutLocked GetEnts dxf
                                         ENT I LOCKED OBJ SS UFLAG)
  (vl-load-com)  

  (defun *error* (msg)
    (and uFlag (vla-EndUndoMark *doc))
    (or (wcmatch (strcase msg) "*BREAK,*CANCEL*,*EXIT*")
        (princ (strcat "\n** Error: " msg " **")))
    (princ))
  

  (defun GetLocked (/ lst)
    (vlax-for lay (vla-get-Layers *doc)
      (and (eq :vlax-true (vla-get-lock lay))
           (setq lst (cons lay lst))
           (vla-put-lock lay :vlax-false)))
    lst)
  

  (defun PutLocked (lst)
    (mapcar
      (function
        (lambda (x)
          (vla-put-lock x :vlax-true))) lst))
  

  (defun GetEnts  (ent)
    (if (setq ent (entnext ent))
      (cons ent (GetEnts ent))))


  (defun dxf (code ent)
    (cdr (assoc code (entget ent))))
  

  (setq *doc (cond (*doc) ((vla-get-ActiveDocument
                             (vlax-get-acad-object))))

        Args (strcase (car Args)))

  (cond (  (and (wcmatch Args "*INSERT,*EXECUTETOOL")
                (setq ent (entlast))

                (eq "AcDbBlockReference"
                    (vla-get-ObjectName
                      (setq obj (vlax-ename->vla-object ent))))
                
                (eq :vlax-true (vla-get-HasAttributes obj)))
         
           (setq uFlag  (not (vla-StartUndoMark *doc))
                 Locked (GetLocked))
         
           (foreach att (append (vlax-invoke obj 'GetAttributes)
                                (vlax-invoke obj 'GetConstantAttributes))
           
             (vla-put-rotation att 0.))
         
           (PutLocked Locked)
           (setq uFlag (vla-EndUndoMark *doc)))

        (  (or (and (wcmatch Args "*MIRROR,*COPY")
                    (setq ss  (ssget "_P" '((0 . "INSERT") (66 . 1)))))

               (and (wcmatch Args "*GRIP_ROTATE,*GRIP_MIRROR")
                    (setq ss (cadr (ssgetfirst)))
                    (eq "INSERT" (dxf 0 (ssname ss 0)))
                    (= (dxf 66 (ssname ss 0)) 1)))

           (if *ZeroLastEnt*
             (foreach x (GetEnts *ZeroLastEnt*)
               (if (and (eq "INSERT" (dxf 0 x)) (= 1 (dxf 66 x)))
                 (ssadd x ss))))

           (setq uFlag  (not (vla-StartUndoMark *doc))
                 Locked (GetLocked) i -1)

           (while (setq ent (ssname ss (setq i (1+ i))))
             (setq obj (vlax-ename->vla-object ent))

             (foreach att (append (vlax-invoke obj 'GetAttributes)
                                  (vlax-invoke obj 'GetConstantAttributes))
               
               (vla-put-rotation att 0.)))

           (PutLocked Locked)
           (setq uFlag (vla-EndUndoMark *doc))))             

  (princ))